# encoding: utf-8
require_relative 'helper'

class TestRoundhouse < Roundhouse::Test
  describe 'json processing' do
    it 'loads json' do
      assert_equal ({"foo" => "bar"}), Roundhouse.load_json("{\"foo\":\"bar\"}")
    end

    it 'dumps json' do
      assert_equal "{\"foo\":\"bar\"}", Roundhouse.dump_json({ "foo" => "bar" })
    end
  end

  describe "redis connection" do
  	it "returns error without creating a connection if block is not given" do
      mock = Minitest::Mock.new
      mock.expect :create, nil #Roundhouse::RedisConnection, create
  		assert_raises(ArgumentError) {
  			Roundhouse.redis
  		}
      assert_raises(MockExpectationError, "create should not be called") do
        mock.verify
      end
  	end
  end

  describe "❨╯°□°❩╯︵┻━┻" do
    before { $stdout = StringIO.new }
    after  { $stdout = STDOUT }

    it "allows angry developers to express their emotional constitution and remedies it" do
      Roundhouse.❨╯°□°❩╯︵┻━┻
      assert_equal "Calm down, yo.\n", $stdout.string
    end
  end

  describe 'lifecycle events' do
    it 'handles invalid input' do
      Roundhouse.options[:lifecycle_events][:startup].clear

      e = assert_raises ArgumentError do
        Roundhouse.on(:startp)
      end
      assert_match(/Invalid event name/, e.message)
      e = assert_raises ArgumentError do
        Roundhouse.on('startup')
      end
      assert_match(/Symbols only/, e.message)
      Roundhouse.on(:startup) do
        1 + 1
      end

      assert_equal 2, Roundhouse.options[:lifecycle_events][:startup].first.call
    end
  end

  describe 'default_worker_options' do
    before do
      @old_options = Roundhouse.default_worker_options
    end
    after  { Roundhouse.default_worker_options = @old_options }

    it 'stringify keys' do
      Roundhouse.default_worker_options = { queue: 'cat'}
      assert_equal 'cat', Roundhouse.default_worker_options['queue']
    end
  end

  describe 'error handling' do
    it 'deals with user-specified error handlers which raise errors' do
      output = capture_logging do
        begin
          Roundhouse.error_handlers << proc {|x, hash|
            raise 'boom'
          }
          cli = Roundhouse::CLI.new
          cli.handle_exception(RuntimeError.new("hello"))
        ensure
          Roundhouse.error_handlers.pop
        end
      end
      assert_includes output, "boom"
      assert_includes output, "ERROR"
    end
  end

  describe 'redis connection' do
    it 'does not continually retry' do
      assert_raises Redis::CommandError do
        Roundhouse.redis do |c|
          raise Redis::CommandError, "READONLY You can't write against a read only slave."
        end
      end
    end

    it 'reconnects if connection is flagged as readonly' do
      counts = []
      Roundhouse.redis do |c|
        counts << c.info['total_connections_received'].to_i
        raise Redis::CommandError, "READONLY You can't write against a read only slave." if counts.size == 1
      end
      assert_equal 2, counts.size
      assert_equal counts[0] + 1, counts[1]
    end
  end
end
