require_relative 'helper'
require 'roundhouse/scheduled'
require 'active_support/core_ext/numeric/time'
require 'active_support/core_ext/integer/time'
require 'active_support/core_ext/date'

class TestScheduling < Roundhouse::Test
  describe 'middleware' do
    before do
      Roundhouse::Client.instance_variable_set(:@default, nil)
      @redis = Minitest::Mock.new
      # Ugh, this is terrible.
      Roundhouse.instance_variable_set(:@redis, @redis)
      def @redis.multi; [yield] * 2 if block_given?; end
      def @redis.with; yield self; end
    end

    after do
      Roundhouse::Client.instance_variable_set(:@default, nil)
      Roundhouse.instance_variable_set(:@redis, REDIS)
    end

    class ScheduledWorker
      include Roundhouse::Worker
      roundhouse_options :queue => :custom_queue
      def perform(x)
      end
    end

    it 'schedules a job via interval' do
      @redis.expect :zadd, true, ['schedule', Array]
      assert ScheduledWorker.perform_in(100, 600, 'mike')
      @redis.verify
    end

    it 'schedules a job in one month' do
      @redis.expect :zadd, true do |key, args|
        assert_equal 'schedule', key
        assert_in_delta 1.month.since.to_f, args[0][0].to_f, 1
      end
      assert ScheduledWorker.perform_in(100, 1.month, 'mike')
      @redis.verify
    end

    it 'schedules a job via timestamp' do
      @redis.expect :zadd, true, ['schedule', Array]
      assert ScheduledWorker.perform_in(100, 5.days.from_now, 'mike')
      @redis.verify
    end

    it 'schedules job right away on negative timestamp/interval' do
      Roundhouse::Monitor.stub :maybe_add_to_rotation, true do
        @redis.expect :lpush, true, ['queue:100', Array]
        assert ScheduledWorker.perform_in(100, -300, 'mike')
      end
      @redis.verify
    end

    it 'schedules multiple jobs at once' do
      @redis.expect :zadd, true, ['schedule', Array]
      assert Roundhouse::Client.push_bulk('queue_id' => 100, 'class' => ScheduledWorker, 'args' => [['mike'], ['mike']], 'at' => 600)
      @redis.verify
    end

    it 'removes the enqueued_at field when scheduling' do
      @redis.expect :zadd, true do |key, args|
        job = Roundhouse.load_json(args.first.last)
        job.key?('created_at') && !job.key?('enqueued_at')
      end
      assert ScheduledWorker.perform_in(100, 1.month, 'mike')
      @redis.verify
    end
  end

end
