require_relative 'helper'
require 'roundhouse'
require 'roundhouse/worker'
require 'roundhouse/rails'

Roundhouse.hook_rails!

class TestTesting < Roundhouse::Test
  describe 'roundhouse testing' do
    describe 'require/load roundhouse/testing.rb' do
      before do
        require 'roundhouse/testing.rb'
      end

      after do
        Roundhouse::Testing.disable!
      end

      it 'enables fake testing' do
        Roundhouse::Testing.fake!
        assert_equal true, Roundhouse::Testing.enabled?
        assert_equal true, Roundhouse::Testing.fake?
      end

      it 'enables fake testing in a block' do
        Roundhouse::Testing.disable!
        assert_equal true, Roundhouse::Testing.disabled?

        Roundhouse::Testing.fake! do
          assert_equal true, Roundhouse::Testing.enabled?
          assert_equal true, Roundhouse::Testing.fake?
        end

        assert_equal false, Roundhouse::Testing.enabled?
        assert_equal false, Roundhouse::Testing.fake?
      end

      it 'disables testing in a block' do
        Roundhouse::Testing.fake!

        Roundhouse::Testing.disable! do
          assert_equal true, Roundhouse::Testing.disabled?
        end

        assert_equal true, Roundhouse::Testing.enabled?
      end
    end

    describe 'require/load roundhouse/testing/inline.rb' do
      before do
        require 'roundhouse/testing/inline.rb'
      end

      after do
        Roundhouse::Testing.disable!
      end

      it 'enables inline testing' do
        Roundhouse::Testing.inline!
        assert_equal true, Roundhouse::Testing.enabled?
        assert_equal true, Roundhouse::Testing.inline?
      end

      it 'enables inline testing in a block' do
        Roundhouse::Testing.disable!
        assert_equal true, Roundhouse::Testing.disabled?

        Roundhouse::Testing.inline! do
          assert_equal true, Roundhouse::Testing.enabled?
          assert_equal true, Roundhouse::Testing.inline?
        end

        assert_equal false, Roundhouse::Testing.enabled?
        assert_equal false, Roundhouse::Testing.inline?
      end
    end
  end
end
