require_relative 'helper'
require 'roundhouse'
require 'roundhouse/worker'
require 'roundhouse/rails'

Roundhouse.hook_rails!

class TestInline < Roundhouse::Test
  describe 'roundhouse inline testing' do
    class InlineError < RuntimeError; end
    class ParameterIsNotString < RuntimeError; end

    class InlineWorker
      include Roundhouse::Worker
      def perform(pass)
        raise ArgumentError, "no jid" unless jid
        raise InlineError unless pass
      end
    end

    class InlineWorkerWithTimeParam
      include Roundhouse::Worker
      def perform(time)
        raise ParameterIsNotString unless time.is_a?(String) || time.is_a?(Numeric)
      end
    end

    before do
      require 'roundhouse/testing/inline.rb'
      Roundhouse::Testing.inline!
    end

    after do
      Roundhouse::Testing.disable!
    end

    it 'stubs the async call when in testing mode' do
      assert InlineWorker.perform_async(100, true)

      assert_raises InlineError do
        InlineWorker.perform_async(100, false)
      end
    end

    it 'stubs the enqueue call when in testing mode' do
      assert Roundhouse::Client.enqueue_to(100, InlineWorker, true)

      assert_raises InlineError do
        Roundhouse::Client.enqueue_to(100, InlineWorker, false)
      end
    end

    it 'stubs the push_bulk call when in testing mode' do
      assert Roundhouse::Client.push_bulk({'queue_id' => 100, 'class' => InlineWorker, 'args' => [[true], [true]]})

      assert_raises InlineError do
        Roundhouse::Client.push_bulk({'queue_id' => 100, 'class' => InlineWorker, 'args' => [[true], [false]]})
      end
    end

    it 'should relay parameters through json' do
      assert Roundhouse::Client.enqueue_to(100, InlineWorkerWithTimeParam, Time.now)
    end
  end
end
